package org.xbib.interlibrary.catalog.builder;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.content.rdf.RouteRdfXContent;

public class CatalogEntityFileRdfOutput implements CatalogEntityRdfOutput {

    private static final Logger logger = Logger.getLogger(CatalogEntityFileRdfOutput.class.getName());

    private final ConverterOutput converterOutput;

    public CatalogEntityFileRdfOutput(ConverterOutput converterOutput) {
        this.converterOutput = converterOutput;
    }

    public ConverterOutput getFileOutput() {
        return converterOutput;
    }

    @Override
    public String getIndex() {
        return "index";
    }

    @Override
    public String getType() {
        return "type";
    }

    @Override
    public RouteRdfXContent.RouteHandler createHandler() {
        return (content, p) -> {
            for (Map.Entry<String, ConverterOutput.Entry> entry : converterOutput.getMap().entrySet()) {
                try {
                    synchronized (converterOutput) {
                        OutputStream outputStream = entry.getValue().getOut();
                        outputStream.write(content.getBytes(StandardCharsets.UTF_8));
                        outputStream.write('\n');
                    }
                } catch (IOException e) {
                    logger.log(Level.SEVERE, e.getMessage(), e);
                }
            }
        };
    }
}
