package org.xbib.interlibrary.catalog.builder;

import java.io.IOException;
import java.util.Iterator;
import org.xbib.interlibrary.api.action.worker.AbstractWorkerPool;
import org.xbib.interlibrary.api.action.worker.DefaultWorkerPoolListener;
import org.xbib.interlibrary.api.action.worker.Worker;
import org.xbib.interlibrary.api.action.worker.WorkerRequest;
import org.xbib.settings.Settings;

public abstract class ConverterWorkerPool<R> extends AbstractWorkerPool<R> {

    protected final Settings settings;

    private final ConverterInput<R> converterInput;

    private final ConverterOutput converterOutput;

    private ConverterMetrics converterMetrics;

    public ConverterWorkerPool(String poolName, Settings settings) {
        super(poolName != null ? poolName : settings.get("name", "unnamed"),
                settings.getAsInt("workers", Runtime.getRuntime().availableProcessors()),
                new DefaultWorkerPoolListener<>());
        this.settings = settings;
        this.converterInput = new ConverterInput<>();
        this.converterOutput = new ConverterOutput();
    }

    public ConverterInput<R> getConverterInput() {
        return converterInput;
    }

    public ConverterOutput getConverterOutput() {
        return converterOutput;
    }

    public ConverterMetrics getConverterMetrics() {
        return converterMetrics;
    }

    @Override
    public Iterator<WorkerRequest<R>> iterator() {
        return getQueue().iterator();
    }

    @Override
    public void afterExecute(Worker<R> worker, WorkerRequest<R> request) {
        if (converterMetrics != null) {
            converterMetrics.mark();
        }
    }

    protected void openInput() throws IOException, InterruptedException {
        converterInput.createInputFromFileMap(settings, this);
    }

    protected void openOutput() throws IOException {
        converterOutput.createOutputFromFileMap(settings);
    }

    protected void openMetrics() throws IOException {
        if (settings.getAsBoolean("metrics.enabled", false)) {
            this.converterMetrics = new ConverterMetrics(settings);
            converterMetrics.openMetrics();
        }
    }

    @Override
    public void close() throws IOException {
        super.close();
        closeMetrics();
        closeInput(getReturnCode());
        closeOutput(getReturnCode());
    }

    protected void closeInput(int returncode) throws IOException {
        // nothing to do
    }

    protected void closeOutput(int returncode) throws IOException {
        if (converterOutput != null) {
            converterOutput.closeFileMap(returncode);
        }
    }

    protected void closeMetrics() throws IOException {
        if (converterMetrics != null) {
            converterMetrics.closeMetrics();
        }
    }
}
