package org.xbib.interlibrary.catalog.builder.doaj;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.xbib.content.rdf.Resource;
import org.xbib.content.rdf.internal.DefaultAnonymousResource;
import org.xbib.content.resource.IRI;
import org.xbib.interlibrary.catalog.entities.ArticleVocabulary;

class DOAJMapper implements ArticleVocabulary {

    private static final Logger logger = Logger.getLogger(DOAJMapper.class.getName());

    public Resource map(String id, Map<String, ?> map) throws IOException {
        Resource r = new DefaultAnonymousResource();
        r.setId(IRI.create(id));
        map(r, null, map);
        return r;
    }

    @SuppressWarnings("unchecked")
    private void map(Resource r, String prefix, Map<String, ?> map) throws IOException {
        for (Map.Entry<String, ?> entry : map.entrySet()) {
            String key = entry.getKey();
            String p = prefix != null ? prefix + "." + key : key;
            Object value = entry.getValue();
            if (value instanceof Map) {
                map(r, p, (Map<String, Object>) value);
            } else if (value instanceof List) {
                for (Object o : (List<String>) value) {
                    if (o instanceof Map) {
                        map(r, p, (Map<String, Object>) o);
                    } else {
                        map(r, p, o.toString());
                    }
                }
            } else {
                if (value != null) {
                    map(r, p, value.toString());
                }
            }
        }
    }

    public void map(Resource r, String path, String value) throws IOException {
        switch (path) {
            case "dc:title" -> r.add(DC_TITLE, value);
            case "dc:identifier" -> {
                if (!value.startsWith("http")) {
                    r.add(DC_IDENTIFIER, value);
                }
            }
            case "dc:language" -> r.add(DC_LANGUAGE, value);
            case "dc:rights" -> r.add(DC_RIGHTS, value);
            case "dc:publisher" -> r.add(DC_PUBLISHER, value);
            case "dc:relation" -> r.add(DC_RELATION, value);
            case "dc:date" -> r.add(DC_DATE, value);
            case "dc:type" -> r.add(DC_TYPE, value);
            case "dc:subject" -> r.add(DC_SUBJECT, value);
            default -> throw new IOException("unable to handle path: " + path);
        }
    }
}
