package org.xbib.interlibrary.catalog.builder.oai;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.content.rdf.RouteRdfXContent;
import org.xbib.interlibrary.catalog.builder.CatalogEntityFileRdfOutput;
import org.xbib.interlibrary.catalog.builder.ConverterOutput;

public class OAICatalogEntityFileRdfOutput extends CatalogEntityFileRdfOutput {

    private static final Logger logger = Logger.getLogger(OAICatalogEntityFileRdfOutput.class.getName());

    private final Object lock = new Object();

    public OAICatalogEntityFileRdfOutput(ConverterOutput converterOutput) {
        super(converterOutput);
    }

    public RouteRdfXContent.RouteHandler createHandler(final String identifier) {
        return (content, p) -> write(identifier, content);
    }

    public void write(String id, String content) {
        if (content == null) {
            logger.log(Level.WARNING, "id = " + id + " content is null?");
            return;
        }
        for (Map.Entry<String, ConverterOutput.Entry> entry : getFileOutput().getMap().entrySet()) {
            try {
                synchronized (lock) {
                    OutputStream outputStream = entry.getValue().getOut();
                    outputStream.write(id.getBytes(StandardCharsets.UTF_8));
                    outputStream.write('\n');
                    outputStream.write(content.getBytes(StandardCharsets.UTF_8));
                    outputStream.write('\n');
                    outputStream.flush();
                }
            } catch (IOException e) {
                logger.log(Level.SEVERE, e.getMessage(), e);
            }
        }
    }
}
