package org.xbib.interlibrary.catalog.builder.util;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class MapHelper {

    private MapHelper() {
    }

    public static <T> T getAnyObject(Map<?, ?> map, String key) {
        return get(map, key);
    }

    public static String getString(Map<?, ?> map, String key) {
        Object object = get(map, key);
        if (object instanceof List) {
            return ((List<?>) object).get(0).toString();
        }
        if (object instanceof Map) {
            return null;
        }
        return object != null ? object.toString() : null;
    }

    public static Integer getInteger(Map<?, ?> map, String key) {
        try {
            Object o = get(map, key);
            return o == null ? null : o instanceof Integer ? (Integer) o : Integer.parseInt(o.toString());
        } catch (NumberFormatException e) {
            return null;
        }
    }

    @SuppressWarnings("unchecked")
    public static Set<Integer> getIntegerSet(Map<String, Object> map, String key) {
        Object o = map.get(key);
        if (o instanceof Collection) {
            return new HashSet<>((Collection<Integer>) o);
        } else if (o != null) {
            return Collections.singleton((Integer) o);
        }
        return Collections.emptySet();
    }

    private static <T> T get(Map<?, ?> map, String key) {
        return get(map, key.split("\\."));
    }

    @SuppressWarnings("unchecked")
    private static <T> T get(Map<?, ?> map, String[] keys) {
        if (map == null) {
            return null;
        }
        String key = keys[0];
        Object o = map.get(key);
        if (o == null) {
            return null;
        }
        if (!(o instanceof List)) {
            o = Collections.singletonList(o);
        }
        List<?> list = (List<?>) o;
        if (keys.length == 1) {
            return (T) list.get(0);
        }
        for (Object oo : list) {
            if (oo instanceof Map) {
                Map<?, ?> m = (Map<?, ?>) oo;
                if (keys.length == 2) {
                    if (m.containsKey(keys[1])) {
                        return (T) m.get(keys[1]);
                    }
                } else {
                    Object ooo = get(m, Arrays.copyOfRange(keys, 1, keys.length));
                    if (ooo != null) {
                        return (T) ooo;
                    }
                }
            } else if (oo instanceof List) {
                List<?> l = (List<?>) oo;
                return (T) l.get(0);
            } else {
                return (T) oo;
            }
        }
        return null;
    }
}
