package org.xbib.interlibrary.catalog.builder.util;

import java.util.Comparator;

/**
 * Perform 'natural order' comparisons of strings.
 * Written by <a href="mailto:natorder@paour.com">Pierre-Luc Paour</a>
 * <p>
 * Based on the C version by Martin Pool, of which this is more or less a straight conversion.
 * Written by <a href="mbp@humbug.org.au">Martin Pool</a>
 *
 * @param <T> parameter type
 */
public class NaturalOrderComparator<T> implements Comparator<T> {

    public NaturalOrderComparator() {
    }

    private static char charAt(String s, int i) {
        return i >= s.length() ? 0 : s.charAt(i);
    }

    @Override
    public int compare(T o1, T o2) {
        String a = o1.toString();
        String b = o2.toString();
        int ia = 0;
        int ib = 0;
        int nza;
        int nzb;
        char ca;
        char cb;
        int result;
        while (true) {
            // only count the number of zeroes leading the last number compared
            nza = nzb = 0;
            ca = charAt(a, ia);
            cb = charAt(b, ib);
            while (Character.isSpaceChar(ca) || ca == '0') {
                if (ca == '0') {
                    nza++;
                } else {
                    nza = 0;
                }
                ca = charAt(a, ++ia);
            }
            while (Character.isSpaceChar(cb) || cb == '0') {
                if (cb == '0') {
                    nzb++;
                } else {
                    nzb = 0;
                }
                cb = charAt(b, ++ib);
            }
            if (Character.isDigit(ca) && Character.isDigit(cb) &&
                    (result = compareRight(a.substring(ia), b.substring(ib))) != 0) {
                return result;
            }
            if (ca == 0 && cb == 0) {
                return nza - nzb;
            }
            if (ca < cb) {
                return -1;
            } else if (ca > cb) {
                return +1;
            }
            ++ia;
            ++ib;
        }
    }

    private int compareRight(String a, String b) {
        int bias = 0;
        int ia = 0;
        int ib = 0;
        while (true) {
            char ca = charAt(a, ia);
            char cb = charAt(b, ib);
            if ((!Character.isDigit(ca) && !Character.isDigit(cb)) || (ca == 0 && cb == 0)) {
                return bias;
            } else if (!Character.isDigit(ca)) {
                return -1;
            } else if (!Character.isDigit(cb)) {
                return +1;
            } else if (ca < cb) {
                if (bias == 0) {
                    bias = -1;
                }
            } else if (ca > cb && bias == 0) {
                bias = 1;
            }
            ia++;
            ib++;
        }
    }
}
